# Print an optspec for argparse to handle cmd's options that are independent of any subcommand.
function __fish_surface_global_optspecs
	string join \n q/quiet h/help V/version
end

function __fish_surface_needs_command
	# Figure out if the current invocation already has a command.
	set -l cmd (commandline -opc)
	set -e cmd[1]
	argparse -s (__fish_surface_global_optspecs) -- $cmd 2>/dev/null
	or return
	if set -q argv[1]
		# Also print the command, so this can be used to figure out what it is.
		echo $argv[1]
		return 1
	end
	return 0
end

function __fish_surface_using_subcommand
	set -l cmd (__fish_surface_needs_command)
	test -z "$cmd"
	and return 1
	contains -- $cmd[1] $argv
end

complete -c surface -n "__fish_surface_needs_command" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_needs_command" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_needs_command" -s V -l version -d 'Print version'
complete -c surface -n "__fish_surface_needs_command" -f -a "dgpu" -d 'Control the discrete GPU'
complete -c surface -n "__fish_surface_needs_command" -f -a "dtx" -d 'Control the latch/dtx-system on the Surface Book 2'
complete -c surface -n "__fish_surface_needs_command" -f -a "profile" -d 'Control or query the current platform profile'
complete -c surface -n "__fish_surface_needs_command" -f -a "status" -d 'Show an overview of the current system status'
complete -c surface -n "__fish_surface_needs_command" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and not __fish_seen_subcommand_from id get-power-state get-runtime-pm set-runtime-pm help" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and not __fish_seen_subcommand_from id get-power-state get-runtime-pm set-runtime-pm help" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and not __fish_seen_subcommand_from id get-power-state get-runtime-pm set-runtime-pm help" -f -a "id" -d 'Get the dGPU PCI device ID'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and not __fish_seen_subcommand_from id get-power-state get-runtime-pm set-runtime-pm help" -f -a "get-power-state" -d 'Get the dGPU PCI power state'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and not __fish_seen_subcommand_from id get-power-state get-runtime-pm set-runtime-pm help" -f -a "get-runtime-pm" -d 'Get the dGPU runtime PM control'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and not __fish_seen_subcommand_from id get-power-state get-runtime-pm set-runtime-pm help" -f -a "set-runtime-pm" -d 'Set the dGPU runtime PM control'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and not __fish_seen_subcommand_from id get-power-state get-runtime-pm set-runtime-pm help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from id" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from id" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from get-power-state" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from get-power-state" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from get-runtime-pm" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from get-runtime-pm" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from set-runtime-pm" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from set-runtime-pm" -s h -l help -d 'Print help (see more with \'--help\')'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from help" -f -a "id" -d 'Get the dGPU PCI device ID'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from help" -f -a "get-power-state" -d 'Get the dGPU PCI power state'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from help" -f -a "get-runtime-pm" -d 'Get the dGPU runtime PM control'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from help" -f -a "set-runtime-pm" -d 'Set the dGPU runtime PM control'
complete -c surface -n "__fish_surface_using_subcommand dgpu; and __fish_seen_subcommand_from help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -f -a "lock" -d 'Lock the latch'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -f -a "unlock" -d 'Unlock the latch'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -f -a "request" -d 'Request latch-open or abort if already in progress'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -f -a "confirm" -d 'Confirm latch-open if detachment in progress'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -f -a "heartbeat" -d 'Send heartbeat if detachment in progress'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -f -a "cancel" -d 'Cancel any detachment in progress'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -f -a "get-base" -d 'Get information about the currently attached base'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -f -a "get-devicemode" -d 'Query the current device operation mode'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -f -a "get-latchstatus" -d 'Query the current latch status'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -f -a "monitor" -d 'Monitor DTX events'
complete -c surface -n "__fish_surface_using_subcommand dtx; and not __fish_seen_subcommand_from lock unlock request confirm heartbeat cancel get-base get-devicemode get-latchstatus monitor help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from lock" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from lock" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from unlock" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from unlock" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from request" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from request" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from confirm" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from confirm" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from heartbeat" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from heartbeat" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from cancel" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from cancel" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from get-base" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from get-base" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from get-devicemode" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from get-devicemode" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from get-latchstatus" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from get-latchstatus" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from monitor" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from monitor" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from help" -f -a "lock" -d 'Lock the latch'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from help" -f -a "unlock" -d 'Unlock the latch'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from help" -f -a "request" -d 'Request latch-open or abort if already in progress'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from help" -f -a "confirm" -d 'Confirm latch-open if detachment in progress'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from help" -f -a "heartbeat" -d 'Send heartbeat if detachment in progress'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from help" -f -a "cancel" -d 'Cancel any detachment in progress'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from help" -f -a "get-base" -d 'Get information about the currently attached base'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from help" -f -a "get-devicemode" -d 'Query the current device operation mode'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from help" -f -a "get-latchstatus" -d 'Query the current latch status'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from help" -f -a "monitor" -d 'Monitor DTX events'
complete -c surface -n "__fish_surface_using_subcommand dtx; and __fish_seen_subcommand_from help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c surface -n "__fish_surface_using_subcommand profile; and not __fish_seen_subcommand_from set get list help" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand profile; and not __fish_seen_subcommand_from set get list help" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand profile; and not __fish_seen_subcommand_from set get list help" -f -a "set" -d 'Set the current platform profile'
complete -c surface -n "__fish_surface_using_subcommand profile; and not __fish_seen_subcommand_from set get list help" -f -a "get" -d 'Get the current platform profile'
complete -c surface -n "__fish_surface_using_subcommand profile; and not __fish_seen_subcommand_from set get list help" -f -a "list" -d 'List all available platform profiles'
complete -c surface -n "__fish_surface_using_subcommand profile; and not __fish_seen_subcommand_from set get list help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c surface -n "__fish_surface_using_subcommand profile; and __fish_seen_subcommand_from set" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand profile; and __fish_seen_subcommand_from set" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand profile; and __fish_seen_subcommand_from get" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand profile; and __fish_seen_subcommand_from get" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand profile; and __fish_seen_subcommand_from list" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand profile; and __fish_seen_subcommand_from list" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand profile; and __fish_seen_subcommand_from help" -f -a "set" -d 'Set the current platform profile'
complete -c surface -n "__fish_surface_using_subcommand profile; and __fish_seen_subcommand_from help" -f -a "get" -d 'Get the current platform profile'
complete -c surface -n "__fish_surface_using_subcommand profile; and __fish_seen_subcommand_from help" -f -a "list" -d 'List all available platform profiles'
complete -c surface -n "__fish_surface_using_subcommand profile; and __fish_seen_subcommand_from help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c surface -n "__fish_surface_using_subcommand status" -s q -l quiet -d 'Keep output quiet'
complete -c surface -n "__fish_surface_using_subcommand status" -s h -l help -d 'Print help'
complete -c surface -n "__fish_surface_using_subcommand help; and not __fish_seen_subcommand_from dgpu dtx profile status help" -f -a "dgpu" -d 'Control the discrete GPU'
complete -c surface -n "__fish_surface_using_subcommand help; and not __fish_seen_subcommand_from dgpu dtx profile status help" -f -a "dtx" -d 'Control the latch/dtx-system on the Surface Book 2'
complete -c surface -n "__fish_surface_using_subcommand help; and not __fish_seen_subcommand_from dgpu dtx profile status help" -f -a "profile" -d 'Control or query the current platform profile'
complete -c surface -n "__fish_surface_using_subcommand help; and not __fish_seen_subcommand_from dgpu dtx profile status help" -f -a "status" -d 'Show an overview of the current system status'
complete -c surface -n "__fish_surface_using_subcommand help; and not __fish_seen_subcommand_from dgpu dtx profile status help" -f -a "help" -d 'Print this message or the help of the given subcommand(s)'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dgpu" -f -a "id" -d 'Get the dGPU PCI device ID'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dgpu" -f -a "get-power-state" -d 'Get the dGPU PCI power state'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dgpu" -f -a "get-runtime-pm" -d 'Get the dGPU runtime PM control'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dgpu" -f -a "set-runtime-pm" -d 'Set the dGPU runtime PM control'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dtx" -f -a "lock" -d 'Lock the latch'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dtx" -f -a "unlock" -d 'Unlock the latch'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dtx" -f -a "request" -d 'Request latch-open or abort if already in progress'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dtx" -f -a "confirm" -d 'Confirm latch-open if detachment in progress'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dtx" -f -a "heartbeat" -d 'Send heartbeat if detachment in progress'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dtx" -f -a "cancel" -d 'Cancel any detachment in progress'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dtx" -f -a "get-base" -d 'Get information about the currently attached base'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dtx" -f -a "get-devicemode" -d 'Query the current device operation mode'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dtx" -f -a "get-latchstatus" -d 'Query the current latch status'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from dtx" -f -a "monitor" -d 'Monitor DTX events'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from profile" -f -a "set" -d 'Set the current platform profile'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from profile" -f -a "get" -d 'Get the current platform profile'
complete -c surface -n "__fish_surface_using_subcommand help; and __fish_seen_subcommand_from profile" -f -a "list" -d 'List all available platform profiles'
